@@ -19,6 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <glib.h>
 #include <dbus/dbus-glib-bindings.h>
 
@@ -28,17 +29,10 @@
 #include "seahorse-key-source.h"
 #include "seahorse-gpgmex.h"
 #include "seahorse-util.h"
+#include "seahorse-libdialogs.h"
 
 #define KEYSET_PATH "/org/gnome/seahorse/keys/%s"
-
-/* Special fields */
-enum {
-    FIELD_DISLAY_NAME,
-    FIELD_CN,
-    FIELD_SPECIAL_MAX
-};
-
-GQuark key_fields[FIELD_SPECIAL_MAX] = { 0 };
+#define KEYSET_PATH_LOCAL "/org/gnome/seahorse/keys/%s/local"
 
 G_DEFINE_TYPE (SeahorseService, seahorse_service, G_TYPE_OBJECT);
 
@@ -49,120 +43,34 @@ copy_to_array (const gchar *type, gpointer dummy, GArray *a)
     g_array_append_val (a, v);
 }
 
-static void
-value_free (gpointer value)
-{
-    g_value_unset ((GValue*)value);
-    g_free (value);
-}
-
-static gboolean
-lookup_key_field (SeahorseKey *skey, guint uid, const gchar *field, GValue *value)
-{
-    GParamSpec *spec;
-    GQuark qfield;
-    gchar *name;
-    
-    /* Special UID fields */
-    if (uid > 0) {
-
-        qfield = g_quark_from_string (field);
-        if (qfield == key_fields[FIELD_DISLAY_NAME]) {
-            name = seahorse_key_get_name (skey, uid);
-            g_value_init (value, G_TYPE_STRING);
-            g_value_take_string (value, name ? name : g_strdup (""));
-            return TRUE;
-
-        } else if (qfield == key_fields[FIELD_CN]) {
-            name = seahorse_key_get_name_cn (skey, uid);
-            g_value_init (value, G_TYPE_STRING);
-            g_value_take_string (value, name ? name : g_strdup (""));
-            return TRUE;
-        }
-    }
-
-    spec = g_object_class_find_property (G_OBJECT_GET_CLASS (skey), field);
-    if (!spec) 
-        return FALSE;
-
-    g_value_init (value, spec->value_type);
-    g_object_get_property (G_OBJECT (skey), field, value);
-    return TRUE; 
-}
-
-void 
+static void 
 add_key_source (SeahorseService *svc, GQuark ktype)
 {
     const gchar *keytype = g_quark_to_string (ktype);
+    SeahorseKeyset *keyset;
     gchar *dbus_id;
     
     /* Check if we have a keyset for this key type, and add if not */
     if (svc->keysets && !g_hash_table_lookup (svc->keysets, keytype)) {
-        SeahorseKeyset *keyset = seahorse_service_keyset_new (ktype);
 
-        /* Register it with DBUS */
+        /* Keyset for all keys */
+        keyset = seahorse_service_keyset_new (ktype, SKEY_LOC_INVALID);
         dbus_id = g_strdup_printf (KEYSET_PATH, keytype);
         dbus_g_connection_register_g_object (seahorse_dbus_server_get_connection (),
                                              dbus_id, G_OBJECT (keyset));    
         g_free (dbus_id);
         
+        /* Keyset for local keys */
+        keyset = seahorse_service_keyset_new (ktype, SKEY_LOC_LOCAL);
+        dbus_id = g_strdup_printf (KEYSET_PATH_LOCAL, keytype);
+        dbus_g_connection_register_g_object (seahorse_dbus_server_get_connection (),
+                                             dbus_id, G_OBJECT (keyset));    
+        g_free (dbus_id);
+        
         g_hash_table_replace (svc->keysets, g_strdup (keytype), keyset);
     }
 }
 
-/* -----------------------------------------------------------------------------
- * PUBLIC METHODS 
- */ 
-
-SeahorseKey*
-seahorse_service_key_from_dbus (const gchar *key, guint *uid)
-{
-    SeahorseKey *skey;
-    gchar **vec;
-    char *t = NULL;
-    
-    vec = g_strsplit (key, ":", 3);
-    if (!vec[0] || !vec[1])
-        return NULL;
-    
-    skey = seahorse_context_find_key (SCTX_APP (), g_quark_from_string (vec[0]), 
-                                      SKEY_LOC_UNKNOWN, vec[1]);
-    
-    if (uid)
-        *uid = 0;
-        
-    /* Parse out the uid */
-    if (skey && vec[2]) {
-        glong l = strtol (vec[2], &t, 10);
-            
-        /* Make sure it's valid */
-        if (*t || l < 0 || l >= seahorse_key_get_num_names (skey))
-            skey = NULL;
-        else if (uid)
-            *uid = (guint)l;
-    }
-    
-    g_strfreev (vec);
-    return skey;
-}
-
-gchar*
-seahorse_service_key_to_dbus (SeahorseKey *skey, guint uid)
-{
-    return seahorse_service_keyid_to_dbus (seahorse_key_get_ktype (skey), 
-                                           seahorse_key_get_keyid (skey), uid);
-}
-
-gchar*
-seahorse_service_keyid_to_dbus (GQuark ktype, const gchar *keyid, guint uid)
-{
-        if (uid == 0)
-        return g_strdup_printf ("%s:%s", g_quark_to_string (ktype), keyid);
-    else
-        return g_strdup_printf ("%s:%s:%d", g_quark_to_string (ktype), keyid, uid);
-    
-}
-
 /* -----------------------------------------------------------------------------
  * DBUS METHODS 
  */
@@ -200,87 +108,6 @@ seahorse_service_get_keyset (SeahorseService *svc, gchar *ktype,
     return TRUE;
 }
 
-gboolean
-seahorse_service_has_key_field (SeahorseService *svc, gchar *key, gchar *field,
-                                gboolean *has, GError **error)
-{
-    SeahorseKey *skey;
-    GParamSpec *spec;
-    GQuark qfield;
-    guint uid;
-    
-    skey = seahorse_service_key_from_dbus (key, &uid);
-    if (!skey) {
-        g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
-                     _("Invalid or unrecognized key: %s"), key);
-        return FALSE;
-    }
-    
-    if (uid > 0) {
-        /* UIDs always have certain fields */
-        qfield = g_quark_from_string (field);
-        if (qfield == key_fields[FIELD_DISLAY_NAME] ||
-            qfield == key_fields[FIELD_CN])
-            return TRUE;
-    }
-    
-    spec = g_object_class_find_property (G_OBJECT_GET_CLASS (skey), field);
-    *has = spec != NULL ? TRUE : FALSE;
-    return TRUE; 
-}
-
-gboolean
-seahorse_service_get_key_field (SeahorseService *svc, gchar *key, gchar *field,
-                                GValue *value, GError **error)
-{
-    SeahorseKey *skey;
-    guint uid;
-
-    skey = seahorse_service_key_from_dbus (key, &uid);
-    if (!skey) {
-        g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
-                     _("Invalid or unrecognized key: %s"), key);
-        return FALSE;
-    }
- 
-    if (!lookup_key_field (skey, uid, field, value)) {
-        g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
-                     _("Invalid field: %s"), field);  
-        return FALSE;
-    }        
-
-    return TRUE; 
-}
-
-gboolean
-seahorse_service_get_key_fields (SeahorseService *svc, gchar *key, gchar **fields,
-                                 GHashTable **values, GError **error)
-{
-    SeahorseKey *skey;
-    GValue *value;
-    guint uid;
-    
-    skey = seahorse_service_key_from_dbus (key, &uid);
-    if (!skey) {
-        g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_INVALID, 
-                     _("Invalid or unrecognized key: %s"), key);
-        return FALSE;
-    }
-
-    *values = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, value_free);
-    
-    while (*fields) {
-        value = g_new0 (GValue, 1);
-        if (lookup_key_field (skey, uid, *fields, value))
-            g_hash_table_insert (*values, g_strdup (*fields), value);
-        else
-            g_free (value);
-        fields++;
-    }
-    
-    return TRUE; 
-}
-
 gboolean
 seahorse_service_import_keys (SeahorseService *svc, gchar *ktype, 
                               gchar *data, gchar ***keys, GError **error)
@@ -313,7 +140,8 @@ seahorse_service_import_keys (SeahorseService *svc, gchar *ktype,
     
     a = g_array_new (TRUE, TRUE, sizeof (gchar*));
     for (l = (GList*)seahorse_operation_get_result (op); l; l = g_list_next (l)) {
-        t = seahorse_service_key_to_dbus (SEAHORSE_KEY (l->data), 0);
+        t = seahorse_context_keyid_to_dbus (SCTX_APP (), 
+                                seahorse_key_get_keyid (SEAHORSE_KEY (l->data)), 0);
         g_array_append_val (a, t);
     }
     
@@ -345,7 +173,7 @@ seahorse_service_export_keys (SeahorseService *svc, gchar *ktype,
     }    
     
     while (*keys) {
-        skey = seahorse_service_key_from_dbus (*keys, 0);
+        skey = seahorse_context_key_from_dbus (SCTX_APP (), *keys, 0);
         
         if (!skey || seahorse_key_get_ktype (skey) != type) {
             gpgme_data_release (gdata);
@@ -397,21 +225,25 @@ seahorse_service_export_keys (SeahorseService *svc, gchar *ktype,
     } 
     
     /* TODO: We should be base64 encoding this */
-    *data = seahorse_util_write_data_to_text (gdata, TRUE);
+    *data = seahorse_util_write_data_to_text (gdata, NULL);
+    gpgmex_data_release (gdata);
     return TRUE;
 }
 
 gboolean
-seahorse_service_match_keys (SeahorseService *svc, gchar *ktype, gint flags, 
-                             gchar **patterns, gchar ***keys, gchar***unmatched,
-                             GError **error)
+seahorse_service_display_notification (SeahorseService *svc, gchar *heading,
+                                       gchar *text, gchar *icon, gboolean urgent, 
+                                       GError **error)
 {
-    /* TODO: Implement match keys */
-    g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_NOTIMPLEMENTED, "TODO");
-    return FALSE;    
+    if (!icon || !icon[0])
+        icon = NULL;
     
+    seahorse_notification_display (heading, text, urgent, icon, NULL);
+    return TRUE;
 }
 
+#if 0
+
 gboolean
 seahorse_service_match_save (SeahorseService *svc, gchar *ktype, gint flags, 
                              gchar **patterns, gchar **keys, GError **error)
@@ -421,14 +253,7 @@ seahorse_service_match_save (SeahorseService *svc, gchar *ktype, gint flags,
     return FALSE;    
 }
 
-gboolean
-seahorse_service_discover_keys (SeahorseService *svc, gchar *ktype, gint flags, 
-                                gchar **patterns, gchar **keys, GError **error)
-{
-    /* TODO: Implement discover keys */
-    g_set_error (error, SEAHORSE_DBUS_ERROR, SEAHORSE_DBUS_ERROR_NOTIMPLEMENTED, "TODO");
-    return FALSE;    
-}
+#endif /* 0 */
 
 /* -----------------------------------------------------------------------------
  * SIGNAL HANDLERS 
@@ -478,10 +303,6 @@ seahorse_service_class_init (SeahorseServiceClass *klass)
     gobject_class = G_OBJECT_CLASS (klass);
     
     gobject_class->dispose = seahorse_service_dispose;
-    
-    /* Some special fields */
-    key_fields[FIELD_DISLAY_NAME] = g_quark_from_static_string ("display-name");
-    key_fields[FIELD_CN] = g_quark_from_static_string ("cn");
 }
 
 static void
