@@ -31,6 +31,7 @@
  * Properties base classes must implement:
  *  ktype: (GQuark) The ktype (ie: SKEY_PGP) of keys originating from this 
  *         key source.
+ *  key-desc: (gchar*) Description for the type of keys originating here.
  *  location: (SeahorseKeyLoc) The location of keys that come from this 
  *         source. (ie: SKEY_LOC_LOCAL, SKEY_LOC_REMOTE)
  *  uri: (gchar*) Only for remote key sources. The full URI of the keyserver 
@@ -60,7 +61,7 @@ typedef struct _SeahorseKeySource {
 
 /* Flags for the get_state method below */
 typedef enum {
-    SKSRC_REMOTE = 0x00000001,
+    SKSRC_REMOTE  = 0x00000001,
     SKSRC_LOADING = 0x00000010
 } SeahorseKeySourceFlags;
 
@@ -80,7 +81,7 @@ typedef struct _SeahorseKeySourceClass {
     /**
      * load
      * @sksrc: The #SeahorseKeySource.
-     * @op: The type of load todo. See SeahorseKeySourceLoad.
+     * @op: The type of load to do. See SeahorseKeySourceLoad.
      * @match: Match text (valid for SKSRC_LOAD_KEY and SKSRC_LOAD_SEARCH ops).
      * 
      * Loads the requested keys, and add the keys to SeahorseContext. 
@@ -88,7 +89,7 @@ typedef struct _SeahorseKeySourceClass {
      * Returns: The load operation.
      */
     SeahorseOperation* (*load) (SeahorseKeySource *sksrc, SeahorseKeySourceLoad op,
-                                const gchar *match);
+                                GQuark keyid, const gchar *match);
     
     /**
      * stop
@@ -104,7 +105,7 @@ typedef struct _SeahorseKeySourceClass {
     /**
      * import
      * @sksrc: The #SeahorseKeySource to import into.
-     * @data: The data to import (not freed).
+     * @data: The data to import (takes ownership).
      *
      * Import keys into the key source. When operation is 'done' a GList of 
      * updated keys may be found as the operation result. 
@@ -127,7 +128,21 @@ typedef struct _SeahorseKeySourceClass {
      */    
     SeahorseOperation* (*export) (SeahorseKeySource *sksrc, GList *keys, 
                                   gboolean complete, gpgme_data_t data);
-                                  
+
+    /**
+     * export_raw
+     * @sksrc: The #SeahorseKeySource to export from.
+     * @keys: A list of key ids to export.
+     * @data: Optional data object to export to (not freed).
+     *
+     * Import keys into the key source. When operation is 'done' the result
+     * of the operation will be a gpgme_data_t 
+     * 
+     * Returns: The export operation
+     */    
+    SeahorseOperation* (*export_raw) (SeahorseKeySource *sksrc, GSList *keyids, 
+                                      gpgme_data_t data);
+
     /**
      * remove
      * @sksrc: The #SeahorseKeySource to delete the key from.
@@ -153,27 +168,39 @@ guint               seahorse_key_source_get_state        (SeahorseKeySource *sks
 
 SeahorseOperation*  seahorse_key_source_load             (SeahorseKeySource *sksrc,
                                                           SeahorseKeySourceLoad load,
+                                                          GQuark keyid,
                                                           const gchar *match);
                                                           
 void                seahorse_key_source_load_sync        (SeahorseKeySource *sksrc,
                                                           SeahorseKeySourceLoad load,
-                                                          const gchar *match);                                                          
+                                                          GQuark keyid,
+                                                          const gchar *match);
 
 void                seahorse_key_source_load_async       (SeahorseKeySource *sksrc,
                                                           SeahorseKeySourceLoad load,
-                                                          const gchar *key);                                                          
+                                                          GQuark keyid,
+                                                          const gchar *match);
 
+/* Takes ownership of |data| */
 SeahorseOperation*  seahorse_key_source_import           (SeahorseKeySource *sksrc,
                                                           gpgme_data_t data);
-                                                          
+
+/* Takes ownership of |data| */
 gboolean            seahorse_key_source_import_sync      (SeahorseKeySource *sksrc,
                                                           gpgme_data_t data,
                                                           GError **err);
 
+SeahorseOperation*  seahorse_key_source_export_keys      (GList *keys, 
+                                                          gpgme_data_t data);
+
 SeahorseOperation*  seahorse_key_source_export           (SeahorseKeySource *sksrc,
                                                           GList *keys,
                                                           gboolean complete,
-                                                          gpgme_data_t data);                        
+                                                          gpgme_data_t data);
+
+SeahorseOperation*  seahorse_key_source_export_raw       (SeahorseKeySource *sksrc, 
+                                                          GSList *keyids, 
+                                                          gpgme_data_t data);
 
 void                seahorse_key_source_stop             (SeahorseKeySource *sksrc);
 
@@ -186,4 +213,7 @@ GQuark              seahorse_key_source_get_ktype        (SeahorseKeySource *sks
 
 SeahorseKeyLoc      seahorse_key_source_get_location     (SeahorseKeySource *sksrc);
 
+GQuark              seahorse_key_source_cannonical_keyid (GQuark ktype, 
+                                                          const gchar *keyid);
+
 #endif /* __SEAHORSE_KEY_SOURCE_H__ */
