@@ -1,7 +1,7 @@
 /*
  * Seahorse
  *
- * Copyright (C) 2004 Nate Nielsen
+ * Copyright (C) 2004 - 2006 Nate Nielsen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,18 +18,24 @@
  * 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
- 
+
+#include "config.h"
 #include <stdlib.h>
 #include <libintl.h>
 #include <gnome.h>
 #include <ldap.h>
 
+#ifdef WITH_SOUP
+#include <libsoup/soup-address.h>
+#endif
+
 #include "config.h"
 #include "seahorse-gpgmex.h"
 #include "seahorse-operation.h"
 #include "seahorse-ldap-source.h"
 #include "seahorse-util.h"
 #include "seahorse-pgp-key.h"
+#include "seahorse-vfs-data.h"
 
 #ifdef WITH_LDAP
 
@@ -288,6 +294,9 @@ escape_ldap_value (const gchar *v)
 typedef gboolean (*OpLDAPCallback)   (SeahorseOperation *op, LDAPMessage *result);
     
 DECLARE_OPERATION (LDAP, ldap)
+#ifdef WITH_SOUP
+    SoupAddress *addr;              /* For async DNS lookup */
+#endif
     SeahorseLDAPSource *lsrc;       /* The source */
     LDAP *ldap;                     /* The LDAP connection */
     int ldap_op;                    /* The current LDAP async msg */
@@ -320,11 +329,23 @@ static void
 seahorse_ldap_operation_dispose (GObject *gobject)
 {
     SeahorseLDAPOperation *lop = SEAHORSE_LDAP_OPERATION (gobject);
-    
+
+#ifdef WITH_SOUP
+    if (lop->addr)
+        g_object_unref (lop->addr);
+    lop->addr = NULL;
+#endif 
+        
     if (lop->lsrc) {
         g_object_unref (lop->lsrc);
         lop->lsrc = NULL;
     }
+    
+    if (lop->ldap_op != -1) {
+        if (lop->ldap)
+            ldap_abandon (lop->ldap, lop->ldap_op);
+        lop->ldap_op = -1;
+    }
 
     if (lop->ldap) {
         ldap_unbind (lop->ldap);
@@ -360,6 +381,13 @@ seahorse_ldap_operation_cancel (SeahorseOperation *operation)
     g_assert (SEAHORSE_IS_LDAP_OPERATION (operation));
     lop = SEAHORSE_LDAP_OPERATION (operation);
     
+#ifdef WITH_SOUP
+    /* This cancels lookup */
+    if (lop->addr)
+        g_object_unref (lop->addr);
+    lop->addr = NULL;
+#endif
+    
     if (lop->ldap_op != -1) {
         if (lop->ldap)
             ldap_abandon (lop->ldap, lop->ldap_op);
@@ -437,7 +465,7 @@ result_callback (SeahorseLDAPOperation *lop)
         g_assert (lop->ldap_cb);
 
         /* Should not be marked as done. */
-        g_assert (!seahorse_operation_is_done (SEAHORSE_OPERATION (lop)));
+        g_assert (seahorse_operation_is_running (SEAHORSE_OPERATION (lop)));
     }    
         
     return ret;
@@ -541,34 +569,61 @@ done_bind_start_info (SeahorseOperation *op, LDAPMessage *result)
     return TRUE;
 }
 
-/* Start an LDAP (bind, server info) request */
-static SeahorseLDAPOperation*
-seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb,
-                               guint total)
+/* Once the DNS name is resolved, we end up here */
+static void
+resolved_callback (gpointer unused, guint status, SeahorseLDAPOperation *lop)
 {
-    SeahorseLDAPOperation *lop;
+    guint port = LDAP_PORT;
     gchar *server = NULL;
     gchar *t;
-
-    g_assert (SEAHORSE_IS_LDAP_SOURCE (lsrc));
     
-    lop = g_object_new (SEAHORSE_TYPE_LDAP_OPERATION, NULL);
-    lop->lsrc = lsrc;
-    g_object_ref (lsrc);
+    g_object_get (lop->lsrc, "key-server", &server, NULL);
+    g_return_if_fail (server && server[0]);
     
-    g_object_get (lsrc, "key-server", &server, NULL);
-    g_return_val_if_fail (server && server[0], NULL);
+    if ((t = strchr (server, ':')) != NULL) {
+        *t = 0;
+        t++;
+        port = atoi (t);
+        if (port <= 0 || port >= G_MAXUINT16) {
+            g_warning ("invalid port number: %s (using default)", t);
+            port = LDAP_PORT;
+        }
+    }
+
+#ifdef WITH_SOUP
     
-    lop->ldap = ldap_init (server, LDAP_PORT);
-    g_return_val_if_fail (lop->ldap != NULL, NULL);
-
-    lop->ldap_cb = done_bind_start_info;
-    lop->chain_cb = cb;
-
-    seahorse_operation_mark_start (SEAHORSE_OPERATION (lop));
+    /* DNS failed */
+    if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
+        seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, 
+            g_error_new (SEAHORSE_ERROR, -1, _("Couldn't resolve address: %s"), server));
+        g_free (server);
+        return;
+    } 
+    
+    g_assert (lop->addr);
+    
+    {
+        /* Now that we've resolved our address, connect via IP */
+        const char *ip;
+        ip = soup_address_get_physical (lop->addr);
+        g_return_if_fail (ip != NULL);
+        
+        lop->ldap = ldap_init (ip, port);
+        g_return_if_fail (lop->ldap != NULL);
+    }
+    
+#else /* WITH_SOUP */
+    
+    /* No async DNS resolve, let libldap handle resolving synchronously */
+    lop->ldap = ldap_init (server, port);
+    g_return_if_fail (lop->ldap != NULL);    
+    
+#endif /* WITH_SOUP */
+    
+    /* The ldap_cb and chain_cb were set in seahorse_ldap_operation_start */
     
     t = g_strdup_printf (_("Connecting to: %s"), server);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0, total);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0.0);
     g_free (t);
 
     g_free (server);
@@ -581,7 +636,55 @@ seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb,
     else   /* This starts looking for results */
         lop->stag = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, 
                         (GSourceFunc)result_callback, lop, NULL);
-        
+}
+
+/* Start an LDAP (bind, server info) request */
+static SeahorseLDAPOperation*
+seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb,
+                               guint total)
+{
+    SeahorseLDAPOperation *lop;
+#ifdef WITH_SOUP
+    gchar *server = NULL;
+    gchar *t;
+#endif
+
+    g_assert (SEAHORSE_IS_LDAP_SOURCE (lsrc));
+    
+    lop = g_object_new (SEAHORSE_TYPE_LDAP_OPERATION, NULL);
+    lop->lsrc = lsrc;
+    g_object_ref (lsrc);
+
+    /* Not used until step after resolve */
+    lop->ldap_cb = done_bind_start_info;
+    lop->chain_cb = cb;
+    
+    seahorse_operation_mark_start (SEAHORSE_OPERATION (lop));
+    
+#ifdef WITH_SOUP
+    
+    /* Try and resolve asynchronously */
+    g_object_get (lsrc, "key-server", &server, NULL);
+    g_return_val_if_fail (server && server[0], NULL);
+
+    if ((t = strchr (server, ':')) != NULL)
+        *t = 0;
+    lop->addr = soup_address_new (server, LDAP_PORT);
+    
+    t = g_strdup_printf (_("Resolving server address: %s"), server);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0.0);
+    g_free (t);
+
+    g_free (server);
+    
+    soup_address_resolve_async (lop->addr, (SoupAddressCallback)resolved_callback, lop);
+    
+#else /* no WITH_SOUP */
+    
+    resolved_callback (NULL, 0, lop);
+    
+#endif
+    
     return lop;
 }
 
@@ -707,7 +810,7 @@ start_search (SeahorseOperation *op, LDAPMessage *result)
     g_return_val_if_fail (filter != NULL, FALSE);
 
     t = (gchar*)g_object_get_data (G_OBJECT (lop), "details");
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0, 0);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), t, 0.0);
     
     sinfo = get_ldap_server_info (lop->lsrc, TRUE);
     lop->ldap_op = ldap_search (lop->ldap, sinfo->base_dn, LDAP_SCOPE_SUBTREE,
@@ -787,6 +890,8 @@ get_callback (SeahorseOperation *op, LDAPMessage *result)
     LDAPServerInfo *sinfo;
     gpgme_data_t data;
     char *message;
+    GError *err = NULL;
+    gboolean ret;
     gchar *key;
     int code;
     int r;
@@ -812,13 +917,16 @@ get_callback (SeahorseOperation *op, LDAPMessage *result)
         data = (gpgme_data_t)seahorse_operation_get_result (SEAHORSE_OPERATION (lop));
         g_return_val_if_fail (data != NULL, FALSE);
         
-        r = gpgme_data_write (data, key, strlen (key));
-        g_return_val_if_fail (r != -1, FALSE);
-
-        r = gpgme_data_write (data, "\n", 1);
-        g_return_val_if_fail (r != -1, FALSE);
-
+        ret = seahorse_vfs_data_write_all (data, key, -1, &err) &&
+              seahorse_vfs_data_write_all (data, "\n", -1, &err);
+        
         g_free (key);
+        
+        if (!ret) {
+            seahorse_operation_mark_done (SEAHORSE_OPERATION (lop), FALSE, err);
+            return FALSE;
+        }
+        
         return TRUE;
 
     /* No more entries, result */
@@ -862,9 +970,9 @@ get_key_from_ldap (SeahorseOperation *op, LDAPMessage *result)
     fprfull = (GSList*)g_object_get_data (G_OBJECT (lop), "fingerprints-full");
 
     l = g_slist_length (fprfull);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), 
-                                      _("Retrieving remote keys..."), 
-                                      l - g_slist_length (fingerprints), l);
+    seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (lop), 
+                                           _("Retrieving remote keys..."), 
+                                           l - g_slist_length (fingerprints), l);
     
     if (fingerprints) {
      
@@ -910,7 +1018,6 @@ start_get_operation_multiple (SeahorseLDAPSource *lsrc, GSList *fingerprints,
                               gpgme_data_t data)
 {
     SeahorseLDAPOperation *lop;
-    gpgme_error_t gerr;
     
     g_assert (g_slist_length (fingerprints) > 0);
     
@@ -923,10 +1030,9 @@ start_get_operation_multiple (SeahorseLDAPSource *lsrc, GSList *fingerprints,
         seahorse_operation_mark_result (SEAHORSE_OPERATION (lop), data, NULL);
     } else {
         /* But when we auto create a data object then we free it */
-        gerr = gpgme_data_new (&data);
-        g_return_val_if_fail (data != NULL, NULL);
+        data = gpgmex_data_new ();
         seahorse_operation_mark_result (SEAHORSE_OPERATION (lop), data, 
-                                        (GDestroyNotify)gpgme_data_release);
+                                        (GDestroyNotify)gpgmex_data_release);
     }
     
     g_object_set_data (G_OBJECT (lop), "fingerprints", fingerprints);
@@ -996,9 +1102,9 @@ send_key_to_ldap (SeahorseOperation *op, LDAPMessage *result)
     keysfull = (GSList*)g_object_get_data (G_OBJECT (lop), "key-data-full");
     
     l = g_slist_length (keysfull);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (lop), 
-                                      _("Sending keys to key server..."), 
-                                      l - g_slist_length (keys), l);
+    seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (lop), 
+                                           _("Sending keys to key server..."), 
+                                           l - g_slist_length (keys), l);
     
     if (keys) {
      
@@ -1069,15 +1175,15 @@ start_send_operation_multiple (SeahorseLDAPSource *lsrc, GSList *keys)
 static void seahorse_ldap_source_class_init (SeahorseLDAPSourceClass *klass);
 
 /* SeahorseKeySource methods */
-static SeahorseOperation*  seahorse_ldap_source_load      (SeahorseKeySource *src,
-                                                           SeahorseKeySourceLoad load,
-                                                           const gchar *match);
-static SeahorseOperation*  seahorse_ldap_source_import    (SeahorseKeySource *sksrc, 
-                                                           gpgme_data_t data);
-static SeahorseOperation*  seahorse_ldap_source_export    (SeahorseKeySource *sksrc, 
-                                                           GList *keys,     
-                                                           gboolean complete,
-                                                           gpgme_data_t data);
+static SeahorseOperation*  seahorse_ldap_source_load       (SeahorseKeySource *src,
+                                                            SeahorseKeySourceLoad load,
+                                                            GQuark keyid,
+                                                            const gchar *match);
+static SeahorseOperation*  seahorse_ldap_source_import     (SeahorseKeySource *sksrc, 
+                                                            gpgme_data_t data);
+static SeahorseOperation*  seahorse_ldap_source_export_raw (SeahorseKeySource *sksrc, 
+                                                            GSList *keyids,
+                                                            gpgme_data_t data);
                                                            
 static SeahorseKeySourceClass *parent_class = NULL;
 
@@ -1110,14 +1216,14 @@ seahorse_ldap_source_class_init (SeahorseLDAPSourceClass *klass)
     key_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
     key_class->load = seahorse_ldap_source_load;
     key_class->import = seahorse_ldap_source_import;
-    key_class->export = seahorse_ldap_source_export;
+    key_class->export_raw = seahorse_ldap_source_export_raw;
 
     parent_class = g_type_class_peek_parent (klass);
 }
 
 static SeahorseOperation*
 seahorse_ldap_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
-                           const gchar *match)
+                           GQuark keyid, const gchar *match)
 {
     SeahorseOperation *op;
     SeahorseLDAPOperation *lop = NULL;
@@ -1125,7 +1231,7 @@ seahorse_ldap_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
     g_assert (SEAHORSE_IS_KEY_SOURCE (src));
     g_assert (SEAHORSE_IS_LDAP_SOURCE (src));
 
-    op = parent_class->load (src, load, match);
+    op = parent_class->load (src, load, keyid, match);
     if (op != NULL)
         return op;
     
@@ -1139,7 +1245,8 @@ seahorse_ldap_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
         
     /* Load a specific key */
     else if (load == SKSRC_LOAD_KEY)
-        lop = start_search_operation_fpr (SEAHORSE_LDAP_SOURCE (src), match);
+        lop = start_search_operation_fpr (SEAHORSE_LDAP_SOURCE (src), 
+                                          seahorse_key_get_rawid (keyid));
     
     g_return_val_if_fail (lop != NULL, NULL);
     seahorse_server_source_take_operation (SEAHORSE_SERVER_SOURCE (src),
@@ -1179,26 +1286,24 @@ seahorse_ldap_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
     lop = start_send_operation_multiple (lsrc, keydata);
     g_return_val_if_fail (lop != NULL, NULL);
     
+    gpgmex_data_release (data);
     return SEAHORSE_OPERATION (lop);
 }
 
 static SeahorseOperation* 
-seahorse_ldap_source_export (SeahorseKeySource *sksrc, GList *keys, 
-                             gboolean complete, gpgme_data_t data)
+seahorse_ldap_source_export_raw (SeahorseKeySource *sksrc, GSList *keyids, 
+                                 gpgme_data_t data)
 {
     SeahorseLDAPOperation *lop;
     SeahorseLDAPSource *lsrc;
-    GSList *fingerprints = NULL;
+    GSList *l, *fingerprints = NULL;
     
     g_assert (SEAHORSE_IS_LDAP_SOURCE (sksrc));
     lsrc = SEAHORSE_LDAP_SOURCE (sksrc);
     
-    for ( ; keys; keys = g_list_next (keys)) {
-        g_assert (SEAHORSE_IS_KEY (keys->data));
-        fingerprints = g_slist_prepend (fingerprints,
-                g_strdup (seahorse_key_get_keyid (SEAHORSE_KEY (keys->data))));
-    }
-    
+    for (l = keyids; l; l = g_slist_next (l)) 
+        fingerprints = g_slist_prepend (fingerprints, 
+            g_strdup (seahorse_key_get_rawid (GPOINTER_TO_UINT (l->data))));
     fingerprints = g_slist_reverse (fingerprints);
 
     lop = start_get_operation_multiple (lsrc, fingerprints, data);
