@@ -69,6 +69,7 @@ G_DEFINE_TYPE (SeahorseOperation, seahorse_operation, G_TYPE_OBJECT);
 static gboolean
 delayed_mark_done (SeahorseOperation *operation)
 {
+    g_assert (SEAHORSE_IS_OPERATION (operation));
     g_signal_emit (operation, signals[DONE], 0);
 
     /* A running operation always refs itself */
@@ -203,6 +204,17 @@ seahorse_operation_new_complete (GError *err)
     return operation;
 }
 
+SeahorseOperation*
+seahorse_operation_new_cancelled ()
+{
+    SeahorseOperation *operation;
+    
+    operation = g_object_new (SEAHORSE_TYPE_OPERATION, NULL);
+    seahorse_operation_mark_start (operation);
+    seahorse_operation_mark_done (operation, TRUE, NULL);
+    return operation;
+}
+
 void
 seahorse_operation_cancel (SeahorseOperation *op)
 {
@@ -565,10 +577,15 @@ seahorse_multi_operation_take (SeahorseMultiOperation* mop, SeahorseOperation *o
     
     mop->operations = seahorse_operation_list_add (mop->operations, op);
     
-    g_signal_connect (op, "done", G_CALLBACK (multi_operation_done), mop);
-    g_signal_connect (op, "progress", G_CALLBACK (multi_operation_progress), mop);
-    
-    multi_operation_progress (op, NULL, -1, mop);
+    if (seahorse_operation_is_running (op)) {
+        g_signal_connect (op, "done", G_CALLBACK (multi_operation_done), mop);
+        g_signal_connect (op, "progress", G_CALLBACK (multi_operation_progress), mop);
+        multi_operation_progress (op, NULL, -1, mop);
+        
+    /* Already done, so handle immediately */
+    } else {
+        multi_operation_done (op, mop);
+    }
 }
 
 /* -----------------------------------------------------------------------------
