@@ -65,6 +65,7 @@ enum {
 enum {
     PROP_0,
     PROP_KEY_TYPE,
+    PROP_KEY_DESC,
     PROP_LOCATION
 };
 
@@ -201,6 +202,10 @@ seahorse_pgp_source_class_init (SeahorsePGPSourceClass *klass)
     g_object_class_install_property (gobject_class, PROP_KEY_TYPE,
         g_param_spec_uint ("key-type", "Key Type", "Key type that originates from this key source.", 
                            0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READABLE));
+                           
+    g_object_class_install_property (gobject_class, PROP_KEY_DESC,
+        g_param_spec_string ("key-desc", "Key Desc", "Description for keys that originate here.",
+                             NULL, G_PARAM_READABLE));
 
     g_object_class_install_property (gobject_class, PROP_LOCATION,
         g_param_spec_uint ("location", "Key Location", "Where the key is stored. See SeahorseKeyLoc", 
@@ -319,6 +324,9 @@ seahorse_pgp_source_get_property (GObject *object, guint prop_id, GValue *value,
     case PROP_KEY_TYPE:
         g_value_set_uint (value, SKEY_PGP);
         break;
+    case PROP_KEY_DESC:
+        g_value_set_string (value, _("PGP Key"));
+        break;
     case PROP_LOCATION:
         g_value_set_uint (value, SKEY_LOC_LOCAL);
         break;
@@ -393,7 +401,6 @@ remove_key_from_context (gpointer kt, SeahorseKey *dummy, SeahorsePGPSource *psr
 {
     /* This function gets called as a GHRFunc on the lctx->checks hashtable. */
     GQuark keyid = GPOINTER_TO_UINT (kt);
-    
     SeahorseKey *skey;
     
     skey = seahorse_context_get_key (SCTX_APP (), SEAHORSE_KEY_SOURCE (psrc), keyid);
@@ -743,7 +750,8 @@ seahorse_load_operation_start (SeahorsePGPSource *psrc, const gchar **pattern, g
             skey = SEAHORSE_KEY (l->data);
             if (secret && seahorse_key_get_etype (skey) != SKEY_PRIVATE) 
                 continue;
-            g_hash_table_insert (lop->checks, GUINT_TO_POINTER (seahorse_key_get_keyid (l->data)), NULL);
+            g_hash_table_insert (lop->checks, GUINT_TO_POINTER (seahorse_key_get_keyid (l->data)), 
+                                              GUINT_TO_POINTER (TRUE));
         }
         
         g_list_free (keys);
