@@ -93,6 +93,8 @@ SeahorseKeyPredicate pred_private = {
 #define SEC_RING "/secring.gpg"
 #define PUB_RING "/pubring.gpg"
 
+static void selection_changed (GtkTreeSelection *notused, SeahorseWidget *swidget);
+
 /* SIGNAL CALLBACKS --------------------------------------------------------- */
 
 /* Quits seahorse */
@@ -121,7 +123,7 @@ get_tab_for_key (SeahorseWidget* swidget, SeahorseKey *skey)
     notebook = GTK_NOTEBOOK (glade_xml_get_widget (swidget->xml, "notebook"));
     g_return_val_if_fail (GTK_IS_NOTEBOOK (notebook), NULL);
 
-    pages = gtk_notebook_get_n_pages(notebook);
+    pages = gtk_notebook_get_n_pages (notebook);
     for(i = 0; i < pages; i++)
     {
         widget = gtk_notebook_get_nth_page (notebook, i);
@@ -175,6 +177,7 @@ set_current_tab (SeahorseWidget *swidget, guint tabid)
         if (get_tab_id (gtk_notebook_get_nth_page (notebook, i)) == tabid)
         {
             gtk_notebook_set_current_page (notebook, i);
+            selection_changed (NULL, swidget);
             return;
         }
     }
@@ -224,7 +227,7 @@ get_selected_keys (SeahorseWidget *swidget)
 {
     GtkTreeView *view = get_current_view (swidget);
     g_return_val_if_fail (view != NULL, NULL);
-    return seahorse_key_store_get_selected_keys (view);
+    return seahorse_key_manager_store_get_selected_keys (view);
 }
 
 static SeahorseKey*
@@ -232,7 +235,7 @@ get_selected_key (SeahorseWidget *swidget, guint *uid)
 {
     GtkTreeView *view = get_current_view (swidget);
     g_return_val_if_fail (view != NULL, NULL);
-    return seahorse_key_store_get_selected_key (view, uid);
+    return seahorse_key_manager_store_get_selected_key (view, uid);
 }
 
 static void
@@ -268,7 +271,7 @@ set_selected_keys (SeahorseWidget *swidget, GList* keys)
         if(tab_lists[i])
             last = i;
 
-        seahorse_key_store_set_selected_keys (view, tab_lists[i]);
+        seahorse_key_manager_store_set_selected_keys (view, tab_lists[i]);
         g_list_free (tab_lists[i]);
     }
     
@@ -276,6 +279,14 @@ set_selected_keys (SeahorseWidget *swidget, GList* keys)
         set_current_tab (swidget, last);
 }
 
+static void
+set_selected_key (SeahorseWidget *swidget, SeahorseKey *skey)
+{
+    GList *keys = g_list_prepend (NULL, skey);
+    set_selected_keys (swidget, keys);
+    g_list_free (keys);
+}
+
 static void
 set_numbered_status (SeahorseWidget *swidget, const gchar *t1, const gchar *t2, guint num)
 {
@@ -494,7 +505,7 @@ copy_done (SeahorseOperation *op, SeahorseWidget *swidget)
     data = (gpgme_data_t)seahorse_operation_get_result (op);
     g_return_if_fail (data != NULL);
     
-    text = seahorse_util_write_data_to_text (data, FALSE);
+    text = seahorse_util_write_data_to_text (data, NULL);
     g_return_if_fail (text != NULL);
     
     atom = gdk_atom_intern ("CLIPBOARD", FALSE);
@@ -643,20 +654,10 @@ sign_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
     SeahorseKey *skey;
     guint uid;
-    GList *keys = NULL;
 
     skey = get_selected_key (swidget, &uid);
-    if (SEAHORSE_IS_PGP_KEY (skey)) {
+    if (SEAHORSE_IS_PGP_KEY (skey))
         seahorse_sign_show (SEAHORSE_PGP_KEY (skey), uid);
-        
-#ifdef WITH_KEYSERVER
-        if (seahorse_gconf_get_boolean(AUTOSYNC_KEY) == TRUE) {
-            keys = g_list_append (keys, skey);
-            seahorse_keyserver_sync (keys);
-            g_list_free(keys);
-        }
-#endif
-    }
 }
 
 static void
@@ -810,12 +811,12 @@ row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *arg2
                SeahorseWidget *swidget)
 {
     GtkTreeView *view = get_current_view (swidget);
-	SeahorseKey *skey;
+    SeahorseKey *skey;
     
     g_return_if_fail (view != NULL);
-	
-	skey = seahorse_key_store_get_key_from_path (view, path, NULL);
-	if (skey != NULL)
+
+    skey = seahorse_key_manager_store_get_key_from_path (view, path, NULL);
+    if (skey != NULL)
         show_properties (skey);
 }
 
@@ -823,22 +824,46 @@ static void
 selection_changed (GtkTreeSelection *notused, SeahorseWidget *swidget)
 {
     GtkTreeView *view;
+    GtkWidget *tab;
     GtkActionGroup *actions;
     GList *keys, *l;
-	SeahorseKey *skey = NULL;
+    SeahorseKey *skey = NULL;
     gint ktype = 0;
     gboolean selected = FALSE;
     gboolean secret = FALSE;
-	gint rows = 0;
+    gint rows = 0;
+    GQuark keyid;
     
     view = get_current_view (swidget);
     if (view != NULL) {
         GtkTreeSelection *selection = gtk_tree_view_get_selection (view);
-    	rows = gtk_tree_selection_count_selected_rows (selection);
-	    selected = rows > 0;
+        rows = gtk_tree_selection_count_selected_rows (selection);
+        selected = rows > 0;
     }
-	
-	if (selected) {
+
+    /* Retrieve currently tracked, and reset tracking */
+    keyid = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (swidget), "track-selected-keyid"));
+    g_object_set_data (G_OBJECT (swidget), "track-selected-keyid", NULL);
+    
+    /* no selection, see if selected key moved to another tab */
+    if (rows == 0 && keyid) {
+        
+        /* Find it */
+        skey = seahorse_context_find_key (SCTX_APP (), keyid, SKEY_LOC_LOCAL);
+        if (skey) {
+            
+            /* If it's still around, then select it */
+            tab = get_tab_for_key (swidget, skey);
+            if (tab && tab != get_current_tab (swidget)) {
+
+                /* Make sure we don't end up in a loop  */
+                g_assert (!g_object_get_data (G_OBJECT (swidget), "track-selected-keyid"));
+                set_selected_key (swidget, skey);
+            }
+        }
+    }
+    
+    if (selected) {
         set_numbered_status (swidget, _("Selected %d key"),
                                       _("Selected %d keys"), rows);
         
@@ -859,8 +884,15 @@ selection_changed (GtkTreeSelection *notused, SeahorseWidget *swidget)
                 ktype = 0;
         }
         
+        /* If one key is selected then mark it down for following across tabs */
+        if (g_list_length (keys) == 1) {
+            skey = SEAHORSE_KEY (keys->data);
+            g_object_set_data (G_OBJECT (swidget), "track-selected-keyid", 
+                                GUINT_TO_POINTER (seahorse_key_get_keyid (skey)));
+        }
+        
         g_list_free (keys);
-	}
+    }
     
     actions = seahorse_widget_find_actions (swidget, "key");
     gtk_action_group_set_sensitive (actions, selected);
@@ -940,7 +972,7 @@ target_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, g
 
 /* Refilter the keys when the filter text changes */
 static void
-filter_changed (GtkWidget *widget, SeahorseKeyStore* skstore)
+filter_changed (GtkWidget *widget, SeahorseKeyManagerStore* skstore)
 {
     const gchar* text = gtk_entry_get_text (GTK_ENTRY (widget));
     g_object_set (skstore, "filter", text, NULL);
@@ -984,6 +1016,62 @@ help_activate (GtkWidget *widget, SeahorseWidget *swidget)
     seahorse_widget_show_help (swidget);
 }
 
+static void
+view_type_column (GtkToggleAction *action, SeahorseWidget *swidget)
+{
+    seahorse_gconf_set_boolean (SHOW_TYPE_KEY,
+                gtk_toggle_action_get_active (action));
+}
+
+static void
+view_expiry_column (GtkToggleAction *action, SeahorseWidget *swidget)
+{
+    seahorse_gconf_set_boolean (SHOW_EXPIRES_KEY,
+                gtk_toggle_action_get_active (action));
+}
+
+static void
+view_validity_column (GtkToggleAction *action, SeahorseWidget *swidget)
+{
+    seahorse_gconf_set_boolean (SHOW_VALIDITY_KEY,
+                gtk_toggle_action_get_active (action));
+}
+
+static void
+view_trust_column (GtkToggleAction *action, SeahorseWidget *swidget)
+{
+    seahorse_gconf_set_boolean (SHOW_TRUST_KEY,
+                gtk_toggle_action_get_active (action));
+}
+
+static void
+gconf_notify (GConfClient *client, guint id, GConfEntry *entry, 
+              SeahorseWidget *swidget)
+{
+    GtkActionGroup *actions;
+    GtkAction *action;
+    const gchar *key;
+    
+    actions = seahorse_widget_find_actions (swidget, "main");
+    g_return_if_fail (actions);
+    key = gconf_entry_get_key (entry);
+
+    if (g_str_equal (SHOW_TRUST_KEY, key))
+        action = gtk_action_group_get_action (actions, "view-trust");
+    else if (g_str_equal (SHOW_TYPE_KEY, key))
+        action = gtk_action_group_get_action (actions, "view-type");
+    else if (g_str_equal (SHOW_EXPIRES_KEY, key))
+        action = gtk_action_group_get_action (actions, "view-expires");
+    else if (g_str_equal (SHOW_VALIDITY_KEY, key))
+        action = gtk_action_group_get_action (actions, "view-validity");
+    else
+        return;
+    
+    g_return_if_fail (action);
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), 
+                        gconf_value_get_bool (gconf_entry_get_value (entry)));
+}
+
 /* BUILDING THE MAIN WINDOW ------------------------------------------------- */
 
 static const GtkActionEntry ui_entries[] = {
@@ -1017,6 +1105,17 @@ static const GtkActionEntry ui_entries[] = {
             N_("Show Seahorse help"), G_CALLBACK (help_activate) }, 
 };
 
+static const GtkToggleActionEntry view_entries[] = {
+    { "view-type", NULL, N_("Key T_ypes"), NULL,
+             N_("Show key type column"), G_CALLBACK (view_type_column), FALSE },
+    { "view-expires", NULL, N_("Key _Expiry"), NULL,
+             N_("Show key expiry column"), G_CALLBACK (view_expiry_column), FALSE },
+    { "view-trust", NULL, N_("Key _Trust"), NULL,
+             N_("Show owner trust column"), G_CALLBACK (view_trust_column), FALSE },
+    { "view-validity", NULL, N_("Key _Validity"), NULL,
+             N_("Show key validity column"), G_CALLBACK (view_validity_column), FALSE },
+};
+
 static const GtkActionEntry key_entries[] = {
     { "key-properties", GTK_STOCK_PROPERTIES, N_("P_roperties"), NULL,
             N_("Show key properties"), G_CALLBACK (properties_activate) }, 
@@ -1050,7 +1149,7 @@ initialize_tab (SeahorseWidget *swidget, const gchar *tabwidget, guint tabid,
                 const gchar *viewwidget, SeahorseKeyPredicate *pred)
 {
     SeahorseKeyset *skset;
-    SeahorseKeyStore *skstore;
+    SeahorseKeyManagerStore *skstore;
     GtkTreeSelection *selection;
     GtkTreeView *view;
     GtkWidget *tab;
@@ -1096,8 +1195,21 @@ seahorse_key_manager_show (SeahorseOperation *op)
     gtk_action_group_set_translation_domain (actions, NULL);
     gtk_action_group_add_actions (actions, ui_entries, 
                                   G_N_ELEMENTS (ui_entries), swidget);
+    gtk_action_group_add_toggle_actions (actions, view_entries,
+                                  G_N_ELEMENTS (view_entries), swidget);
     seahorse_widget_add_actions (swidget, actions);
     
+    /* Setup the initial values for the view toggles */
+    action = gtk_action_group_get_action (actions, "view-type");
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), seahorse_gconf_get_boolean (SHOW_TYPE_KEY));
+    action = gtk_action_group_get_action (actions, "view-trust");
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), seahorse_gconf_get_boolean (SHOW_TRUST_KEY));
+    action = gtk_action_group_get_action (actions, "view-expires");
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), seahorse_gconf_get_boolean (SHOW_EXPIRES_KEY));
+    action = gtk_action_group_get_action (actions, "view-validity");
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), seahorse_gconf_get_boolean (SHOW_VALIDITY_KEY));
+    seahorse_gconf_notify_lazy (LISTING_SCHEMAS, (GConfClientNotifyFunc)gconf_notify, swidget, swidget);
+    
     /* Actions that are allowed on all keys */
     actions = gtk_action_group_new ("key");
     gtk_action_group_set_translation_domain (actions, NULL);
