@@ -28,6 +28,7 @@
 #include "seahorse-ssh-operation.h"
 #include "seahorse-vfs-data.h"
 #include "seahorse-gpgmex.h"
+#include "seahorse-gtkstock.h"
 
 #define NOTEBOOK "notebook"
 
@@ -38,23 +39,125 @@ do_main (SeahorseWidget *swidget)
     SeahorseSSHKey *skey;
     GtkWidget *widget;
     gchar *text;
+    const gchar *template;
 
     key = SEAHORSE_KEY_WIDGET (swidget)->skey;
     skey = SEAHORSE_SSH_KEY (key);
 
-    widget = glade_xml_get_widget (swidget->xml, "comment-label");
-    if (widget) {
-        text = seahorse_key_get_display_name (key);
-        gtk_label_set_text (GTK_LABEL (widget), text);  
-        g_free (text);
-    }
+    /* Image */
+    widget = seahorse_widget_get_widget (swidget, "key-image");
+    if (widget)
+        gtk_image_set_from_stock (GTK_IMAGE (widget), SEAHORSE_STOCK_KEY_SSH, GTK_ICON_SIZE_DIALOG);
 
-    widget = glade_xml_get_widget (swidget->xml, "fingerprint-label");
+    /* Name and title */
+    text = seahorse_key_get_display_name (key);
+    widget = seahorse_widget_get_widget (swidget, "comment-entry");
+    if (widget)
+        gtk_entry_set_text (GTK_ENTRY (widget), text);
+    widget = seahorse_widget_get_top (swidget);
+    gtk_window_set_title (GTK_WINDOW (widget), text);
+    g_free (text);
+
+    /* Key id */
+    widget = glade_xml_get_widget (swidget->xml, "id-label");
     if (widget) {
-        text = seahorse_key_get_fingerprint (key);  
+        text = seahorse_key_get_display_id (key);
         gtk_label_set_text (GTK_LABEL (widget), text);
         g_free (text);
     }
+    
+    /* Put in message */
+    widget = seahorse_widget_get_widget (swidget, "trust-message");
+    g_return_if_fail (widget != NULL);
+    template = gtk_label_get_label (GTK_LABEL (widget));
+    text = g_strdup_printf (template, g_get_user_name ());
+    gtk_label_set_markup (GTK_LABEL (widget), text);
+    g_free (text);
+    
+    /* Setup the check */
+    widget = seahorse_widget_get_widget (swidget, "trust-check");
+    g_return_if_fail (widget != NULL);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
+                                  seahorse_key_get_trust (key) >= SEAHORSE_VALIDITY_FULL);
+}
+
+static void
+comment_activate (GtkWidget *entry, SeahorseWidget *swidget)
+{
+    SeahorseKey *key;
+    SeahorseSSHKey *skey;
+    SeahorseKeySource *sksrc;
+    SeahorseOperation *op;
+    const gchar *text;
+    gchar *comment;
+    GError *err = NULL;
+    
+    key = SEAHORSE_KEY_WIDGET (swidget)->skey;
+    skey = SEAHORSE_SSH_KEY (key);
+    sksrc = seahorse_key_get_source (key);
+    g_return_if_fail (SEAHORSE_IS_SSH_SOURCE (sksrc));
+    
+    text = gtk_entry_get_text (GTK_ENTRY (entry));
+    
+    /* Make sure not the same */
+    if (skey->keydata->comment && g_utf8_collate (text, skey->keydata->comment) == 0)
+        return;
+
+    gtk_widget_set_sensitive (entry, FALSE);
+    
+    comment = g_strdup (text);
+    op = seahorse_ssh_operation_rename (SEAHORSE_SSH_SOURCE (sksrc), skey, comment);
+    g_free (comment);
+    
+    /* This is usually a quick operation */
+    seahorse_operation_wait (op);
+    
+    if (!seahorse_operation_is_successful (op)) {
+        seahorse_operation_steal_error (op, &err);
+        seahorse_util_handle_error (err, _("Couldn't rename key."));
+        gtk_entry_set_text (GTK_ENTRY (entry), skey->keydata->comment ? skey->keydata->comment : "");
+    }
+    
+    gtk_widget_set_sensitive (entry, TRUE);
+}
+
+static gboolean
+comment_focus_out (GtkWidget* widget, GdkEventFocus *event, SeahorseWidget *swidget)
+{
+    comment_activate (widget, swidget);
+    return FALSE;
+}
+
+static void 
+trust_toggled (GtkToggleButton *button, SeahorseWidget *swidget)
+{
+    SeahorseKeySource *sksrc;
+    SeahorseOperation *op;
+    SeahorseKey *key;
+    SeahorseSSHKey *skey;
+    gboolean authorize;
+    GError *err = NULL;
+
+    key = SEAHORSE_KEY_WIDGET (swidget)->skey;
+    skey = SEAHORSE_SSH_KEY (key);
+    sksrc = seahorse_key_get_source (key);
+    g_return_if_fail (SEAHORSE_IS_SSH_SOURCE (sksrc));
+    
+    authorize = gtk_toggle_button_get_active (button);
+    gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
+    
+    op = seahorse_ssh_operation_authorize (SEAHORSE_SSH_SOURCE (sksrc), skey, authorize);
+    g_return_if_fail (op);
+    
+    /* A very fast op, so just wait */
+    seahorse_operation_wait (op);
+    
+    if (!seahorse_operation_is_successful (op)) {
+        seahorse_operation_steal_error (op, &err);
+        seahorse_util_handle_error (err, _("Couldn't change authorization for key."));
+    }
+    
+    gtk_widget_set_sensitive (GTK_WIDGET (button), TRUE);
 }
 
 static void
@@ -69,6 +172,7 @@ passphrase_done (SeahorseOperation *op, SeahorseWidget *swidget)
     }
     
     w = glade_xml_get_widget (swidget->xml, "passphrase-button");
+    g_return_if_fail (w != NULL);
     gtk_widget_set_sensitive (w, TRUE);
 }
 
@@ -83,6 +187,7 @@ passphrase_button_clicked (GtkWidget *widget, SeahorseWidget *swidget)
     g_assert (SEAHORSE_IS_SSH_KEY (skey));
 
     w = glade_xml_get_widget (swidget->xml, "passphrase-button");
+    g_return_if_fail (w != NULL);
     gtk_widget_set_sensitive (w, FALSE);
     
     op = seahorse_ssh_operation_change_passphrase (SEAHORSE_SSH_KEY (skey));
@@ -154,10 +259,11 @@ do_details (SeahorseWidget *swidget)
     key = SEAHORSE_KEY_WIDGET (swidget)->skey;
     skey = SEAHORSE_SSH_KEY (key);
 
-    widget = glade_xml_get_widget (swidget->xml, "id-label");
+    widget = glade_xml_get_widget (swidget->xml, "fingerprint-label");
     if (widget) {
-        label = seahorse_key_get_short_keyid (key); 
-        gtk_label_set_text (GTK_LABEL (widget), label);
+        text = seahorse_key_get_fingerprint (key); 
+        gtk_label_set_text (GTK_LABEL (widget), text);
+        g_free (text);
     }
 
     widget = glade_xml_get_widget (swidget->xml, "algo-label");
@@ -168,7 +274,7 @@ do_details (SeahorseWidget *swidget)
 
     widget = glade_xml_get_widget (swidget->xml, "location-label");
     if (widget) {
-        label = seahorse_ssh_key_get_filename (skey, TRUE);
+        label = seahorse_ssh_key_get_location (skey);
         gtk_label_set_text (GTK_LABEL (widget), label);  
     }
 
@@ -241,11 +347,26 @@ seahorse_ssh_key_properties_new (SeahorseSSHKey *skey)
 
     do_main (swidget);
     do_details (swidget);
+    
+    glade_xml_signal_connect_data (swidget->xml, "trust_toggled", 
+                                   G_CALLBACK (trust_toggled), swidget);
+    
+    widget = seahorse_widget_get_widget (swidget, "comment-entry");
+    g_return_if_fail (widget != NULL);
+    g_signal_connect (widget, "activate", G_CALLBACK (comment_activate), swidget);
+    g_signal_connect (widget, "focus-out-event", G_CALLBACK (comment_focus_out), swidget);
 
-    glade_xml_signal_connect_data (swidget->xml, "export_button_clicked",
-                                   G_CALLBACK (export_button_clicked), swidget);        
-    glade_xml_signal_connect_data (swidget->xml, "passphrase_button_clicked",
-                                   G_CALLBACK (passphrase_button_clicked), swidget);        
+    if (seahorse_key_get_etype (key) == SKEY_PRIVATE) {
+        glade_xml_signal_connect_data (swidget->xml, "export_button_clicked",
+                                       G_CALLBACK (export_button_clicked), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "passphrase_button_clicked",
+                                       G_CALLBACK (passphrase_button_clicked), swidget);
+        
+    /* A public key only */
+    } else {
+        seahorse_widget_set_visible (swidget, "passphrase-button", FALSE);
+        seahorse_widget_set_visible (swidget, "export-button", FALSE);
+    }
 
     if (swidget)
         seahorse_widget_show (swidget);
